	CODE32
	AREA int_handler, CODE, READWRITE

    EXPORT MemCpy8
    EXPORT calcd
    EXPORT mandelbrot
    EXPORT mul32x32_64
    
    EXPORT pos
    EXPORT lonx
    EXPORT lony
    EXPORT runx
    EXPORT runy
    
    EXPORT d

    
START_ZOOM  EQU     768

pos
ident       DCB 0
            DCB 0
lonx        DCD (START_ZOOM<<16)
lony        DCD (START_ZOOM<<16)
runx        DCD (START_ZOOM<<16)
runy        DCD (START_ZOOM<<16)
d           DCD 0
            DCD 0
scrx        DCD 69905    ;;(1/240)<<24
scry        DCD 104858   ;;(1/160)<<24

;;;;;;;;;;;;;;;;;;
MemCpy8 ;r0 = src address, r1 = dest address, r2 = byte count

movebyte
    subs r2, r2, #1
    ldrb r3,[r0,r2]
    strb r3,[r1,r2]

    bgt  movebyte
    bx   lr
;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;
mul32x32_64
    smull r3,r2,r0,r1
    mov r2,r2, lsl #8
    orr r0, r2, r3, lsr #24
    bx      lr
;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;
calcd
    ;d.re=mul32x32_64((pos.runx-pos.lonx),69905);
    ;d.im=mul32x32_64((pos.runy-pos.lony),104858);
    ldr     r0, runx
    ldr     r1, lonx
    sub     r0,r0,r1
    ldr     r1, scrx
    smull   r2,r3,r0,r1
    mov     r3,r3, lsl #8
    orr     r0,r3,r2, lsr #24
    ldr     r1, =d
    str     r0,[r1,#0]

    ldr     r0, runy
    ldr     r2, lony
    sub     r0,r0,r2
    ldr     r2, scry
    smull   r2,r3,r0,r2
    mov     r3,r3, lsl #8
    orr     r0,r3,r2, lsr #24
    str     r0,[r1,#4]
    
    bx      lr
    


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mandelbrot 
    ;;;;;;;;;;;;;;;register auf stack schieben...
    stmfd sp!,{r4-r11}
    
    ;;;;;;r0 = iy
    mov     r0,#0  ;clear
    
yloop
    ;;;;;;r1 = ix
    mov     r1,#0
xloop

    ;;initialisieren auf 0
    mov     r3, #0 ;;;k1
    ;;;; r4 wird K2... spter
    mov     r5, #0 ;x.re
    mov     r6, #0 ;x.im
    
    ;;r7 = c.re
    ldr     r9,=d
    ldr     r7,[r9,#0] ;;Realteil von d laden
    mul     r7,r1,r7   ;;mit ix multiplizieren
    ldr     r10,lonx
    add     r7,r7,r10  ;;loax addieren

    ;;r8 = c.im
    ldr     r8,[r9,#4] ;;Imaginrteil von d laden
    mul     r8,r0,r8   ;;mit iy multiplizieren
    ldr     r9,lony
    add     r8,r8,r9   ;;loay addieren
    
    
    
mandelpx1
    ;t.im = (mul32x32_64(x.re,x.im)<<1) +c.im;
    smull   r9,r10,r5,r6
    mov     r10,r10, lsl #9
    orr     r9,r10,r9, lsr #23
    add     r9, r9, r8   ;;i von c addieren (r8)
    

    ;t.re = (mul32x32_64(x.re,x.re)) -
    ;       (mul32x32_64(x.im,x.im)) +c.re;

    ;mul32x32_64(x.im,x.im)
    smull   r11,r10, r6,r6 ;;;r4:r12 = r2
    mov     r10, r10, lsl #8
    orr     r10, r10, r11, lsr #24
    
    mov     r6,r9 ;;;Ergebnis von der ersten Rechnung in x.im speichern
                  ;;;r9 wieder frei...    

    ;mul32x32_64(x.re,x.re)
    smull   r9,r11, r5,r5
    mov     r11, r11, lsl #8
    orr     r9, r11, r9, lsr #24
    
    sub     r9, r9, r10
    add     r5,r9,r7  ;;R5 = realteil von X

  
    add     r3,r3,#1
    cmp     r3, #127
    beq     finishpx1
    
    
    smull   r11,r9,r5,r5
    smull   r11,r10,r6,r6
    add     r9,r10,r9
    cmp     r9,#(4<<16)
    bgt     finishpx1  ;;;Wert zu gross...

    b       mandelpx1   ;;von neuen...

finishpx1
    ;;;r5 - Ergebnis fr ersten Pixel

    add     r1,r1,#1 ;;nchster pixel (ix++)
    mov     r4, #0      ;;zhlvariable fr zweiten pixel
    mov     r5, #0   ;;x.re auf 0
    mov     r6, #0   ;;x.im auf 0

    ;;r7 = c.re
    ldr     r9,=d
    ldr     r7,[r9,#0] ;;Realteil von d laden
    mul     r7,r1,r7   ;;mit ix multiplizieren
    ldr     r10,lonx
    add     r7,r7,r10  ;;loax addieren

  ;;;zweiten pixel loop
mandelpx2


    ;t.im = (mul32x32_64(x.re,x.im)<<1) +c.im;
    smull   r9,r10,r5,r6
    mov     r10,r10, lsl #9
    orr     r9,r10,r9, lsr #23
    add     r9, r9, r8   ;;c.i addieren (r8)
    

    ;t.re = (mul32x32_64(x.re,x.re)) -
    ;       (mul32x32_64(x.im,x.im)) +c.re;

    ;mul32x32_64(x.im,x.im)
    smull   r11,r10, r6,r6 ;;;r4:r12 = r2
    mov     r10, r10, lsl #8
    orr     r10, r10, r11, lsr #24
    
    mov     r6,r9 ;;;Ergebnis von der ersten Rechnung in x.im speichern
                  ;;;r9 wieder frei...    

    ;mul32x32_64(x.re,x.re)
    smull   r9,r11, r5,r5
    mov     r11, r11, lsl #8
    orr     r9, r11, r9, lsr #24
    
    sub     r9, r9, r10
    add     r5,r9,r7  ;;R5 = realteil von X

    add     r3,r3,#(1<<8)
    cmp     r3, #(127<<8)
    bgt     finishpx2
    
    smull   r11,r9,r5,r5
    smull   r11,r10,r6,r6
    
    add     r9,r10,r9
    cmp     r9,#(4<<16)
    bgt     finishpx2  ;;;Wert zu gross...

    
    b       mandelpx2
    
finishpx2



    ;;VideoBuffer16[((y)*120)+((x)>>1)]=(((c2)<<8) | c1)
    mov     r5,#0x6000000
    add     r5,r5,r0,lsl#8
    sub     r5,r5,r0,lsl#4
    strh    r3,[r5,r1]
    
    
    add     r1,r1,#1
    ;x check
    cmp     r1,#240
    blt     xloop    
    
    ;y check
    add     r0,r0,#1
    cmp     r0,#160
    blt     yloop

    ;;;;;;;;;;;;;;;register wiederherstellen
    ldmfd   sp!,{r4-r11} 
    
    bx      lr


	END